@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2007-2015 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2007-2018 Jakub Wilk <jwilk@jwilk.net>
 #
 # This file is part of python-djvulibre.
 #
@@ -16,6 +16,7 @@
 import array
 import errno
 import os
+import re
 import shutil
 import subprocess
 import sys
@@ -79,6 +80,7 @@ from tools import (
     assert_is,
     assert_is_instance,
     assert_list_equal,
+    assert_multi_line_equal,
     assert_raises,
     assert_raises_regex,
     assert_raises_str,
@@ -93,8 +95,6 @@ from tools import (
     wildcard_import,
     # Python 2/3 compat:
     b,
-    bytes,
-    maxsize,
     py3k,
     u,
     unicode,
@@ -109,18 +109,16 @@ else:
 
 def run(*cmd, **kwargs):
     stdin = kwargs.pop('stdin', None)
-    env = {}
+    env = dict(os.environ)
     for key, value in kwargs.items():
         if key.isupper():
             env[key] = value
             continue
         raise TypeError('{key!r} is an invalid keyword argument for this function'.format(key=key))
-    def preexec_fn():
-        os.environ.update(env)
     kwargs = dict(
-        preexec_fn=preexec_fn,
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE,
+        env=env,
     )
     if stdin is not None:
         kwargs.update(stdin=subprocess.PIPE)
@@ -136,11 +134,11 @@ def create_djvu(commands='', sexpr=''):
         commands += '\nset-ant\n{sexpr}\n.\n'.format(sexpr=sexpr)
     file = tempfile.NamedTemporaryFile(prefix='test', suffix='djvu')
     file.seek(0)
-    file.write(bytes([
-        0x41, 0x54, 0x26, 0x54, 0x46, 0x4f, 0x52, 0x4d, 0x00, 0x00, 0x00, 0x22, 0x44, 0x4a, 0x56, 0x55,
-        0x49, 0x4e, 0x46, 0x4f, 0x00, 0x00, 0x00, 0x0a, 0x00, 0x01, 0x00, 0x01, 0x18, 0x00, 0x2c, 0x01,
-        0x16, 0x01, 0x53, 0x6a, 0x62, 0x7a, 0x00, 0x00, 0x00, 0x04, 0xbc, 0x73, 0x1b, 0xd7,
-    ]))
+    file.write(
+        b'\x41\x54\x26\x54\x46\x4F\x52\x4D\x00\x00\x00\x22\x44\x4A\x56\x55'
+        b'\x49\x4E\x46\x4F\x00\x00\x00\x0A\x00\x01\x00\x01\x18\x00\x2C\x01'
+        b'\x16\x01\x53\x6A\x62\x7A\x00\x00\x00\x04\xBC\x73\x1B\xD7'
+    )
     file.flush()
     (stdout, stderr) = run('djvused', '-s', file.name, stdin=commands.encode(locale_encoding))
     assert_equal(stdout, ''.encode(locale_encoding))
@@ -172,8 +170,7 @@ class test_documents:
         path = '__nonexistent__'
         try:
             os.stat(path)
-        except OSError:
-            _, ex, _ = sys.exc_info()
+        except OSError as ex:
             c_message = ex.args[1]
         else:
             raise OSError(errno.EEXIST, os.strerror(errno.EEXIST), path)
@@ -200,8 +197,7 @@ class test_documents:
         try:
             with interim_locale(LC_ALL='ja_JP.UTF-8'):
                 os.stat(path)
-        except OSError:
-            _, ex, _ = sys.exc_info()
+        except OSError as ex:
             c_message = ex.args[1]
         else:
             raise OSError(errno.EEXIST, os.strerror(errno.EEXIST), path)
@@ -321,8 +317,8 @@ class test_documents:
         assert_equal(len(document.pages), 2)
         assert_equal(len(document.files), 3)
         (stdout0, stderr0) = run('djvudump', original_filename, LC_ALL='C')
-        assert_equal(stderr0, b(''))
-        stdout0 = stdout0.replace(b('\r\n'), b('\n'))
+        assert_equal(stderr0, b'')
+        stdout0 = stdout0.replace(b'\r\n', b'\n')
         tmpdir = tempfile.mkdtemp()
         try:
             tmp = open(os.path.join(tmpdir, 'tmp.djvu'), 'wb')
@@ -332,8 +328,8 @@ class test_documents:
             assert_false(job.is_error)
             tmp.close()
             (stdout, stderr) = run('djvudump', tmp.name, LC_ALL='C')
-            assert_equal(stderr, b(''))
-            stdout = stdout.replace(b('\r\n'), b('\n'))
+            assert_equal(stderr, b'')
+            stdout = stdout.replace(b'\r\n', b'\n')
             assert_equal(stdout, stdout0)
         finally:
             shutil.rmtree(tmpdir)
@@ -347,14 +343,14 @@ class test_documents:
             assert_false(job.is_error)
             tmp.close()
             stdout, stderr = run('djvudump', tmp.name, LC_ALL='C')
-            assert_equal(stderr, b(''))
-            stdout = stdout.replace(b('\r\n'), b('\n'))
-            stdout0 = stdout0.split(b('\n'))
-            stdout = stdout.split(b('\n'))
-            stdout[4] = stdout[4].replace(b(' (1)'), b(''))
+            assert_equal(stderr, b'')
+            stdout = stdout.replace(b'\r\n', b'\n')
+            stdout0 = stdout0.split(b'\n')
+            stdout = stdout.split(b'\n')
+            stdout[4] = stdout[4].replace(b' (1)', b'')
             assert_equal(len(stdout), 10)
             assert_equal(stdout[3:-1], stdout0[4:10])
-            assert_equal(stdout[-1], b(''))
+            assert_equal(stdout[-1], b'')
         finally:
             shutil.rmtree(tmpdir)
             tmp = None
@@ -366,16 +362,16 @@ class test_documents:
             assert_true(job.is_done)
             assert_false(job.is_error)
             (stdout, stderr) = run('djvudump', tmpfname, LC_ALL='C')
-            assert_equal(stderr, b(''))
-            stdout = stdout.replace(b('\r\n'), b('\n'))
-            stdout = stdout.split(b('\n'))
+            assert_equal(stderr, b'')
+            stdout = stdout.replace(b'\r\n', b'\n')
+            stdout = stdout.split(b'\n')
             stdout0 = (
-                [b('      shared_anno.iff -> shared_anno.iff')] +
+                [b'      shared_anno.iff -> shared_anno.iff'] +
                 [b('      p{n:04}.djvu -> p{n:04}.djvu'.format(n=n)) for n in range(1, 3)]
             )
             assert_equal(len(stdout), 7)
             assert_equal(stdout[2:-2], stdout0)
-            assert_equal(stdout[-1], b(''))
+            assert_equal(stdout[-1], b'')
         finally:
             shutil.rmtree(tmpdir)
         tmpdir = tempfile.mkdtemp()
@@ -386,13 +382,13 @@ class test_documents:
             assert_true(job.is_done)
             assert_false(job.is_error)
             (stdout, stderr) = run('djvudump', tmpfname, LC_ALL='C')
-            stdout = stdout.replace(b('\r\n'), b('\n'))
-            assert_equal(stderr, b(''))
-            stdout = stdout.split(b('\n'))
+            stdout = stdout.replace(b'\r\n', b'\n')
+            assert_equal(stderr, b'')
+            stdout = stdout.split(b'\n')
             assert_equal(len(stdout), 5)
-            assert_equal(stdout[2], b('      shared_anno.iff -> shared_anno.iff'))
-            assert_equal(stdout[3], b('      p0001.djvu -> p0001.djvu'))
-            assert_equal(stdout[-1], b(''))
+            assert_equal(stdout[2], b'      shared_anno.iff -> shared_anno.iff')
+            assert_equal(stdout[3], b'      p0001.djvu -> p0001.djvu')
+            assert_equal(stdout[-1], b'')
         finally:
             shutil.rmtree(tmpdir)
 
@@ -408,42 +404,27 @@ class test_documents:
         assert_equal(document.type, DOCUMENT_TYPE_BUNDLED)
         assert_equal(len(document.pages), 2)
         assert_equal(len(document.files), 3)
-        tmp = tempfile.NamedTemporaryFile()
-        try:
+        with tempfile.NamedTemporaryFile() as tmp:
             job = document.export_ps(tmp.file)
             assert_equal(type(job), SaveJob)
             assert_true(job.is_done)
             assert_false(job.is_error)
             stdout, stderr = run('ps2ascii', tmp.name, LC_ALL='C')
-            assert_equal(stderr, b(''))
-            assert_equal(stdout, b('\x0c') * 2)
-        finally:
-            tmp.close()
-
-        tmp = tempfile.NamedTemporaryFile()
-        try:
+            assert_equal(stderr, b'')
+            stdout = re.sub(br'[\x00\s]+', b' ', stdout)
+            assert_equal(stdout, b' ')
+        with tempfile.NamedTemporaryFile() as tmp:
             job = document.export_ps(tmp.file, pages=(0,), text=True)
             assert_equal(type(job), SaveJob)
             assert_true(job.is_done)
             assert_false(job.is_error)
             stdout, stderr = run('ps2ascii', tmp.name, LC_ALL='C')
-            assert_equal(stderr, b(''))
-            stdout = stdout.split(b('\n'))
-            stdout = [b(' ').join(line.split()) for line in stdout]
-            assert_equal(stdout, [
-                b(''),
-                b(''),
-                b('1 Lorem ipsum'),
-                b('Optio reprehenderit molestias amet aliquam, similique doloremque fuga labore'),
-                b('voluptatum voluptatem, commodi culpa voluptas, officia tenetur expedita quidem hic'),
-                b(''),
-                b('repellat molestiae quis accusamus dolores repudiandae, quidem in ad voluptas'),
-                b('eligendi maiores placeat ex consectetur at tenetur amet.'),
-                b(''),
-                b('1'),
-            ])
-        finally:
-            del tmp
+            assert_equal(stderr, b'')
+            stdout = stdout.decode('ASCII')
+            stdout = re.sub(r'[\x00\s]+', ' ', stdout)
+            stdout = ' '.join(stdout.split()[:3])
+            expected = '1 Lorem ipsum'
+            assert_multi_line_equal(stdout, expected)
 
 class test_pixel_formats():
 
@@ -460,9 +441,9 @@ class test_pixel_formats():
         assert_repr(pf, "djvu.decode.PixelFormatRgb(byte_order = 'BGR', bpp = 24)")
 
     def test_rgb_mask(self):
-        pf = PixelFormatRgbMask(0xff, 0xf00, 0x1f000, 0, 16)
+        pf = PixelFormatRgbMask(0xFF, 0xF00, 0x1F000, 0, 16)
         assert_repr(pf, "djvu.decode.PixelFormatRgbMask(red_mask = 0x00ff, green_mask = 0x0f00, blue_mask = 0xf000, xor_value = 0x0000, bpp = 16)")
-        pf = PixelFormatRgbMask(0xff000000, 0xff0000, 0xff00, 0xff, 32)
+        pf = PixelFormatRgbMask(0xFF000000, 0xFF0000, 0xFF00, 0xFF, 32)
         assert_repr(pf, "djvu.decode.PixelFormatRgbMask(red_mask = 0xff000000, green_mask = 0x00ff0000, blue_mask = 0x0000ff00, xor_value = 0x000000ff, bpp = 32)")
 
     def test_grey(self):
@@ -539,20 +520,20 @@ class test_page_jobs():
             page_job.render(RENDER_COLOR, (0, 0, 10, 10), (0, 0, 10, 10), PixelFormatRgb(), -1)
 
         with assert_raises_regex(MemoryError, r'\AUnable to allocate [0-9]+ bytes for an image memory\Z'):
-            x = int((maxsize//2) ** 0.5)
+            x = int((sys.maxsize // 2) ** 0.5)
             page_job.render(RENDER_COLOR, (0, 0, x, x), (0, 0, x, x), PixelFormatRgb(), 8)
 
         s = page_job.render(RENDER_COLOR, (0, 0, 10, 10), (0, 0, 4, 4), PixelFormatGrey(), 1)
-        assert_equal(s, bytes([0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0xff, 0xff, 0xff, 0xa4, 0xff, 0xff, 0xff, 0xb8]))
+        assert_equal(s, b'\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xEF\xFF\xFF\xFF\xA4\xFF\xFF\xFF\xB8')
 
-        buffer = array.array('B', bytes([0]))
+        buffer = array.array('B', b'\0')
         with assert_raises_str(ValueError, 'Image buffer is too small (16 > 1)'):
             page_job.render(RENDER_COLOR, (0, 0, 10, 10), (0, 0, 4, 4), PixelFormatGrey(), 1, buffer)
 
-        buffer = array.array('B', bytes([0] * 16))
+        buffer = array.array('B', b'\0' * 16)
         assert_is(page_job.render(RENDER_COLOR, (0, 0, 10, 10), (0, 0, 4, 4), PixelFormatGrey(), 1, buffer), buffer)
         s = array_tobytes(buffer)
-        assert_equal(s, bytes([0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0xff, 0xff, 0xff, 0xa4, 0xff, 0xff, 0xff, 0xb8]))
+        assert_equal(s, b'\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xEF\xFF\xFF\xFF\xA4\xFF\xFF\xFF\xB8')
 
 class test_thumbnails:
 
@@ -572,15 +553,15 @@ class test_thumbnails:
         assert_is(pixels, None)
         (w, h, r), pixels = thumbnail.render((5, 5), PixelFormatGrey())
         assert_equal((w, h, r), (5, 3, 5))
-        assert_equal(pixels[:15], bytes([0xff, 0xeb, 0xa7, 0xf2, 0xff, 0xff, 0xbf, 0x86, 0xbe, 0xff, 0xff, 0xe7, 0xd6, 0xe7, 0xff]))
-        buffer = array.array('B', bytes([0]))
+        assert_equal(pixels[:15], b'\xFF\xEB\xA7\xF2\xFF\xFF\xBF\x86\xBE\xFF\xFF\xE7\xD6\xE7\xFF')
+        buffer = array.array('B', b'\0')
         with assert_raises_str(ValueError, 'Image buffer is too small (25 > 1)'):
             (w, h, r), pixels = thumbnail.render((5, 5), PixelFormatGrey(), buffer=buffer)
-        buffer = array.array('B', bytes([0] * 25))
+        buffer = array.array('B', b'\0' * 25)
         (w, h, r), pixels = thumbnail.render((5, 5), PixelFormatGrey(), buffer=buffer)
         assert_is(pixels, buffer)
         s = array_tobytes(buffer[:15])
-        assert_equal(s, bytes([0xff, 0xeb, 0xa7, 0xf2, 0xff, 0xff, 0xbf, 0x86, 0xbe, 0xff, 0xff, 0xe7, 0xd6, 0xe7, 0xff]))
+        assert_equal(s, b'\xFF\xEB\xA7\xF2\xFF\xFF\xBF\x86\xBE\xFF\xFF\xE7\xD6\xE7\xFF')
 
 def test_jobs():
 
@@ -650,7 +631,7 @@ class test_streams:
         finally:
             message.stream.close()
         with assert_raises_str(IOError, 'I/O operation on closed file'):
-            message.stream.write(b('eggs'))
+            message.stream.write(b'eggs')
         message = document.get_message()
         assert_equal(type(message), DocInfoMessage)
         outline = document.outline
@@ -820,8 +801,8 @@ class test_sexpr:
             Expression(
                 [Symbol('page'), 0, 0, 2550, 3300,
                     '2 Hyperlinks \n'
-                    '2.1 local \n'
-                    + u('→1 \n') +
+                    '2.1 local \n' +
+                    u('→1 \n') +
                     '2.2 remote \nhttp://jwilk.net/ \n'
                     '2 \n'
                 ]
