@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2007-2015 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2007-2018 Jakub Wilk <jwilk@jwilk.net>
 #
 # This file is part of python-djvulibre.
 #
@@ -13,11 +13,13 @@
 # or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 # more details.
 
-import collections
 import codecs
+import collections
 import copy
 import errno
 import io
+import os
+import shutil
 import tempfile
 
 import pickle
@@ -126,7 +128,7 @@ class test_int_expressions():
 
     def test_unpickle(self):
         # pickle as generated by python-djvulibre 0.3.3:
-        p = b("cdjvu.sexpr\n_expression_from_string\np0\n(S'42'\np1\ntp2\nRp3\n.")
+        p = b"cdjvu.sexpr\n_expression_from_string\np0\n(S'42'\np1\ntp2\nRp3\n."
         x = pickle.loads(p)
         self.t(42, x)
 
@@ -224,7 +226,7 @@ class test_symbol_expressions():
         # __hash__():
         assert_equal(
             hash(x),
-            hash(bname.strip(b('|')))
+            hash(bname.strip(b'|'))
         )
         # pickle:
         assert_pickle_equal(x)
@@ -537,10 +539,20 @@ class test_expression_parser():
             Expression.from_stream(42)
 
     def test_bad_file_io(self):
+        if os.name == 'nt':
+            raise SkipTest('not implemented on Windows')
+        path = '/proc/self/mem'
+        try:
+            os.stat(path)
+        except OSError as exc:
+            raise SkipTest('{exc.filename}: {exc.strerror}'.format(exc=exc))
         with open('/proc/self/mem') as fp:
             with assert_raises(IOError) as ecm:
                 Expression.from_stream(fp)
-        assert_equal(ecm.exception.errno, errno.EIO)
+        assert_in(
+            ecm.exception.errno,
+            (errno.EIO, errno.EFAULT)
+        )
 
     if py3k:
         def test_bad_unicode_io(self):
@@ -583,11 +595,15 @@ class test_expression_parser_ascii():
             self.t(fp)
 
     def test_codecs_io(self):
-        with tempfile.NamedTemporaryFile(mode='w+b') as bfp:
-            with codecs.open(bfp.name, mode='w+', encoding='UTF-16-LE') as fp:
+        tmpdir = tempfile.mkdtemp()
+        try:
+            path = os.path.join(tmpdir, 'tmp')
+            with codecs.open(path, mode='w+', encoding='UTF-16-LE') as fp:
                 fp.write(u(self.expr))
                 fp.seek(0)
                 self.t(fp)
+        finally:
+            shutil.rmtree(tmpdir)
 
     def test_file_io_binary(self):
         with tempfile.TemporaryFile(mode='w+b') as fp:
@@ -612,7 +628,12 @@ class test_expression_writer():
 
     def test_bad_file_io(self):
         ecm = None
-        fp = open('/dev/full', 'w', buffering=2)
+        path = '/dev/full'
+        try:
+            os.stat(path)
+        except OSError as exc:
+            raise SkipTest('{exc.filename}: {exc.strerror}'.format(exc=exc))
+        fp = open(path, 'w', buffering=2)
         expr = Expression(23)
         try:
             with assert_raises(IOError) as ecm:
@@ -685,18 +706,26 @@ class test_expression_writer_ascii():
             assert_equal(fp.read(), self.urepr)
 
     def test_codecs_io_text_7(self):
-        with tempfile.NamedTemporaryFile(mode='w+b') as bfp:
-            with codecs.open(bfp.name, mode='w+', encoding='UTF-16-LE') as fp:
+        tmpdir = tempfile.mkdtemp()
+        try:
+            path = os.path.join(tmpdir, 'tmp')
+            with codecs.open(path, mode='w+', encoding='UTF-16-LE') as fp:
                 self.expr.print_into(fp)
                 fp.seek(0)
                 assert_equal(fp.read(), self.repr)
+        finally:
+            shutil.rmtree(tmpdir)
 
     def test_codecs_io_text_8(self):
-        with tempfile.NamedTemporaryFile(mode='w+b') as bfp:
-            with codecs.open(bfp.name, mode='w+', encoding='UTF-16-LE') as fp:
+        tmpdir = tempfile.mkdtemp()
+        try:
+            path = os.path.join(tmpdir, 'tmp')
+            with codecs.open(path, mode='w+', encoding='UTF-16-LE') as fp:
                 self.expr.print_into(fp, escape_unicode=False)
                 fp.seek(0)
                 assert_equal(fp.read(), u(self.urepr))
+        finally:
+            shutil.rmtree(tmpdir)
 
     def test_file_io_binary_7(self):
         with tempfile.TemporaryFile(mode='w+b') as fp:
